from Numeric import arange, indices, ceil, concatenate, take, floor, asarray
from umath import arctan2
import types
import cPickle
GG = globals


class NDgrid:
    def __getitem__(self,key):
        try:
	    size = []
	    for k in range(len(key)):
	        step = key[k].step
                if step == None:
                    step = 1.0
                size.append((key[k].stop - key[k].start)/(step*1.0))
	    nn = indices(size,'d')
	    for k in range(len(size)):
                step = key[k].step
                if step == None:
                    step = 1.0
                nn[k] = (nn[k]+key[k].start/(1.0*step))*step
	    return nn
        except:
            if key.start is None:
               return arange(0,key.stop,key.step)
            else:
               return arange(key.start, key.stop,key.step)
	    
    def __getslice__(self,i,j):
        return arange(i,j)

    def __len__(self):
        return 0


def linspace(start,stop,num,endpoint=1):
    """y = linspace(start,stop,num,endpoint=1)

    return num evenly spaced samples from start to stop.  If endpoint=1 then
    last sample is stop.
    """
    if endpoint:
        step = (stop-start)/float((num-1))
        return arange(start,stop+step,step),step
    else:
        step = (stop-start)/float(num)
        return arange(start,stop,step),step

def round(arr):
    return floor(arr+0.5)

def fftshift(x,axes=None):
    """y = fftshift(x,axes=<all>)

    return a shifted version of x (useful for obtaining centered spectra).
    This function swaps "half-spaces" for all axes listed (defaults to all)
    """
    ndim = len(x.shape)
    if axes == None:
        axes = range(ndim)
    y = x
    for k in axes:
        N = x.shape[k]
        p2 = int(ceil(N/2.0))
        mylist = concatenate((arange(p2,N),arange(p2)))
        y = take(y,mylist,k)
    return y

def ifftshift(x,axes=None):
    """y = ifftshift(x,axes=<all>)

    undoes the effect of fftshift.
    """
    ndim = len(x.shape)
    if axes == None:
        axes = range(ndim)
    y = x
    for k in axes:
        N = x.shape[k]
        p2 = int(floor(N/2.0))
        mylist = concatenate((arange(p2,N),arange(p2)))
        y = take(y,mylist,k)
    return y


def angle(x):
    """y = angle(x)

    return the phase angle of the elements of a complex array.
    """
    return arctan2(x.imag,x.real)


def freqs(N,sample=1.0):
    """freq = freqs(N,sample=1.0)

    return the frequency bins in cycles/unit (with zero at the start) given a
    window length N and a sample spacing.
    """
    sample = float(sample)
    return concatenate((arange(0,(N+1)/2,1,'d'),arange(-(N-1)/2,0,1,'d')))/N/sample

freqz = freqs


def cont_ft(gn,fr,dT=1.0):
    """cont_ft(gn,fr,dT=1.0) computes the DFT of gn at frequencies fr.  If the gn are
    alias-free samples of a continuous time function then inputing the spacing dT will
    give the properly scaled, continuous Fourier spectrum.
    """
    trans_kernel = exp(2j*pi*fr[:,NewAxis]*dT*arange(len(gn)))
    return dT*dot(trans_kernel,gn)


def r1array(x):
    if type(x) in [types.IntType, types.FloatType, types.ComplexType]:
        return asarray([x])
    else:
        return asarray(x)


def objsave(file, allglobals, *args):
    """Pickle the part of a dictionary containing the argument list into file.

    Syntax:  objsave(file, globals(), obj1, obj2, ... )
    """
    fid = open(file,'w')
    savedict = {}
    for key in allglobals.keys():
        inarglist = 0
        for obj in args:
            if allglobals[key] is obj:
                inarglist = 1
                break
        if inarglist:
            savedict[key] = obj
    cPickle.dump(savedict,fid,1)
    fid.close()
        


def objload(file, allglobals):
    """Load a previously pickled dictionary and insert into given dictionary.

    Syntax:  objload(file, globals())
    """
    fid = open(file,'r')
    savedict = cPickle.load(fid)
    allglobals.update(savedict)
    fid.close()





